@@ -11,9 +11,9 @@ import traceback
 import glob
 import threading
 import shutil
-import re
+import regex
 from typing import Tuple, List, Dict, Optional, Callable, Set, Any, Union
-from contextlib import ExitStack
+from contextlib import ExitStack, suppress
 
 from .. import Error, Type, Env, Value, StdLib, Tree, Expr, _util
 from .._util import (
@@ -173,9 +173,15 @@ def run_local_task(
                 )
 
                 # interpolate command
+                setattr(
+                    stdlib,
+                    "_placeholder_regex",
+                    regex.compile(cfg["task_runtime"]["placeholder_regex"], flags=regex.POSIX),
+                )  # hack to pass regex to WDL.Expr.Placeholder._eval
                 command = _util.strip_leading_whitespace(
                     task.command.eval(container_env, stdlib).value
                 )[1]
+                delattr(stdlib, "_placeholder_regex")
                 logger.debug(_("command", command=command.strip()))
 
                 # process command & container through plugins
@@ -327,6 +333,19 @@ def _eval_task_inputs(
     posix_inputs: Env.Bindings[Value.Base],
     container: "runtime.task_container.TaskContainer",
 ) -> Env.Bindings[Value.Base]:
+    # Preprocess inputs: if None value is supplied for an input declared with a default but without
+    # the ? type quantifier, remove the binding entirely so that the default will be used. In
+    # contrast, if the input declaration has an -explicitly- optional type, then we'll allow the
+    # supplied None to override any default.
+    input_decls = task.available_inputs
+    posix_inputs = posix_inputs.filter(
+        lambda b: not (
+            isinstance(b.value, Value.Null)
+            and b.name in input_decls
+            and input_decls[b.name].expr
+            and not input_decls[b.name].type.optional
+        )
+    )
 
     # Map all the provided input File & Directory paths to in-container paths
     container.add_paths(_fspaths(posix_inputs))
@@ -599,6 +618,28 @@ def _eval_task_outputs(
     env: Env.Bindings[Value.Base],
     container: "runtime.task_container.TaskContainer",
 ) -> Env.Bindings[Value.Base]:
+    stdout_file = os.path.join(container.host_dir, "stdout.txt")
+    with suppress(FileNotFoundError):
+        if os.path.getsize(stdout_file) > 0:
+            # If the task produced nonempty stdout that isn't used in the WDL outputs, generate a
+            # courtesy log message directing user where to find it
+            stdout_used = False
+            expr_stack = [outp.expr for outp in task.outputs]
+            while expr_stack:
+                expr = expr_stack.pop()
+                assert isinstance(expr, Expr.Base)
+                if isinstance(expr, Expr.Apply) and expr.function_name == "stdout":
+                    stdout_used = True
+                else:
+                    expr_stack.extend(expr.children)
+            if not stdout_used:
+                logger.notice(  # pyre-ignore
+                    _(
+                        "command stdout unused; consider output `File cmd_out = stdout()`"
+                        " or redirect command to stderr log >&2",
+                        stdout_file=stdout_file,
+                    )
+                )
 
     # helper to rewrite File/Directory from in-container paths to host paths
     def rewriter(v: Union[Value.File, Value.Directory], output_name: str) -> str:
@@ -732,7 +773,7 @@ def link_outputs(
                 sum(
                     1
                     for b in v.value
-                    if re.fullmatch("[-_a-zA-Z0-9][-_a-zA-Z0-9.]*", str(b[0])) is None
+                    if regex.fullmatch("[-_a-zA-Z0-9][-_a-zA-Z0-9.]*", str(b[0])) is None
                 )
                 == 0
             )
