@@ -812,6 +812,29 @@ class MiscRegressionTests(RunnerTestCase):
         for fn in outp["files_out"]:
             assert os.stat(fn).st_uid == euid
 
+    def test_placeholder_regex(self):
+        wdl = """
+        version 1.1
+        task vulnerable {
+            input {
+                String s
+            }
+            command <<<
+                echo 'Hello, ~{s}'
+            >>>
+            output {
+                String out = read_string(stdout())
+            }
+        }
+        """
+        self.assertEqual(self._run(wdl, {"s": "Alice"})["out"], "Hello, Alice")
+        malicious = "'; exit 42; echo '"
+        self._run(wdl, {"s": malicious}, expected_exception=WDL.runtime.CommandFailed)
+        cfg = WDL.runtime.config.Loader(logging.getLogger(self.id()), [])
+        cfg.override({"task_runtime": {"placeholder_regex": "[^']*"}})
+        self._run(wdl, {"s": malicious}, cfg=cfg, expected_exception=WDL.Error.InputError)
+        cfg.override({"task_runtime": {"placeholder_regex": "[0-9A-Za-z:/._-]*"}})
+        self._run(wdl, {"s": malicious}, cfg=cfg, expected_exception=WDL.Error.InputError)
 
 class TestInlineDockerfile(RunnerTestCase):
     @log_capture()
@@ -899,3 +922,65 @@ class TestAbbreviatedCallInput(RunnerTestCase):
         assert sum("18.04" in msg for msg in outputs["results"]) == 2
         outputs = self._run(caller, {"message": "hello", "docker": "ubuntu:focal"})
         assert sum("20.04" in msg for msg in outputs["results"]) == 2
+
+
+class TestImplicitlyOptionalInputWithDefault(RunnerTestCase):
+    def test_workflow(self):
+        src = R"""
+        version 1.1
+        workflow contrived {
+            input {
+                String a = "Alice" + select_first([b, "Carol"])
+                String? b = "Bob"
+            }
+            output {
+                Array[String?] results = [a, b]
+            }
+        }
+        """
+        outp = self._run(src, {})
+        self.assertEqual(outp["results"], ["AliceBob", "Bob"])
+        outp = self._run(src, {"a": "Alyssa"})
+        self.assertEqual(outp["results"], ["Alyssa", "Bob"])
+        outp = self._run(src, {"b": "Bas"})
+        self.assertEqual(outp["results"], ["AliceBas", "Bas"])
+        outp = self._run(src, {"b": None})
+        self.assertEqual(outp["results"], ["AliceCarol", None])
+        outp = self._run(src, {"a": None, "b": None})
+        self.assertEqual(outp["results"], ["AliceCarol", None])
+
+    def test_task(self):
+        caller = R"""
+        version 1.1
+        workflow caller {
+            input {
+                String? a
+                String? b
+            }
+            call contrived {
+                input:
+                a = a, b = b
+            }
+            output {
+                Array[String?] results = contrived.results
+            }
+        }
+        task contrived {
+            input {
+                String a = "Alice" + select_first([b, "Carol"])
+                String? b = "Bob"
+            }
+            command {}
+            output {
+                Array[String?] results = [a, b]
+            }
+        }
+        """
+        outp = self._run(caller, {})
+        self.assertEqual(outp["results"], ["AliceCarol", None])
+        outp = self._run(caller, {"a": None, "b": None})
+        self.assertEqual(outp["results"], ["AliceCarol", None])
+        outp = self._run(caller, {"b": "Bas"})
+        self.assertEqual(outp["results"], ["AliceBas", "Bas"])
+        outp = self._run(caller, {"a": "Alyssa"})
+        self.assertEqual(outp["results"], ["Alyssa", None])
